"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const express = require("express");
const compression = require("compression");
const bodyParser = require("body-parser");
const methodOverride = require("method-override");
const cors = require("cors");
const validall_1 = require("validall");
const codes_1 = require("tools-box/fetch/codes");
const object_from_map_1 = require("tools-box/object/object-from-map");
const url_1 = require("tools-box/url");
let initialized = false;
function initExpress(app, service, negateRoot = false) {
    if (!initialized)
        initialized = true;
    else
        return;
    const contract = service._contract;
    if (service.env === 'prod')
        app.use(compression());
    app.use((req, res, next) => {
        service.logger.info(`request ${req.url} - method ${req.method} - ip ${req.ip}`);
        let start = Date.now();
        res.once('finish', () => {
            service.logger.info(`response ${req.url} - status ${res.statusCode} - in ${(Date.now() - start) / 1000}s`);
        });
        next();
    });
    app.use(methodOverride());
    app.use(cors());
    app.use(bodyParser.urlencoded({ extended: true }));
    app.use(bodyParser.json({ limit: `${service.bodyQuota}mb` }));
    if ('onInit' in service)
        service.onInit(app);
    if ('onRequest' in service)
        app.use(service.onRequest);
    if (!negateRoot) {
        app.use(`${contract.path}`, createRouter(express.Router(), service));
    }
    else {
        createRouter(app, service, negateRoot);
    }
    app.use((req, res, next) => {
        if ('onInvalidRoute' in service) {
            service.onInvalidRoute(req, res, next);
        }
        else {
            res.status(codes_1.CODES.NOT_FOUND).json({ error: 'service not found' });
        }
    });
    app.use((err, req, res, next) => {
        if ('onError' in service)
            service.onError(err, req, res);
        else {
            service.logger.error(err.message, err);
            res.status(codes_1.CODES.UNKNOWN_ERROR).json({ error: service.error(err.message, err) });
        }
    });
    return app;
}
exports.initExpress = initExpress;
function validateRequestHeaders(handlerOptions) {
    return (req, res, next) => {
        if (handlerOptions.inputType && handlerOptions.inputType !== req.headers['content-type']) {
            return res.status(codes_1.CODES.BAD_REQUEST).json({ error: { message: `invalid input type: ${req.headers['content-type']} - expected: ${handlerOptions.inputType}` } });
        }
        if (handlerOptions.inputSchema) {
            let err = validall_1.validate(req.body, object_from_map_1.objFromMap(req.body, {}, handlerOptions.inputSchema, { ignoreKeys: true }));
            if (err) {
                return res.status(codes_1.CODES.BAD_REQUEST).json({ error: { message: err.message, original: err } });
            }
        }
        if (handlerOptions.querySchema) {
            let err = validall_1.validate(req.query, object_from_map_1.objFromMap(req.query, {}, handlerOptions.querySchema, { ignoreKeys: true }));
            if (err) {
                return res.status(codes_1.CODES.BAD_REQUEST).json({ error: { message: err.message, original: err } });
            }
        }
        next();
    };
}
function createRouter(router, service, negateRoot = false) {
    let middlewares = service._middlewares || null;
    if (middlewares !== null) {
        for (let middlewareName in middlewares) {
            let middleware = middlewares[middlewareName];
            if (!(middlewareName in service))
                continue;
            let requestHeadersValidator = validateRequestHeaders(middleware);
            if (middleware.path) {
                router.use(url_1.URL.Clean(`/${middleware.path}`), requestHeadersValidator, (req, res, next) => service[middlewareName](req, res, next));
            }
            else {
                router.use(requestHeadersValidator, (req, res, next) => service[middlewareName](req, res, next));
            }
        }
    }
    let routes = service._routes || null;
    if (routes !== null) {
        for (let routeName in routes) {
            let route = routes[routeName];
            if (!(route.name in service))
                continue;
            let method = route.method.toLowerCase();
            let requestHeadersValidator = validateRequestHeaders(route);
            service.logger.info(`creating Route: ${method} - ${negateRoot ? '' : service._contract.path}${route.path}`);
            router[method](url_1.URL.Clean(`/${route.path}`), route.bodyQuota ? bodyParser.json({ limit: `${route.bodyQuota || 1}mb` }) : (req, res, next) => next(), requestHeadersValidator, service.authorize(routeName), (req, res, next) => service[route.name](req, res, next));
        }
    }
    return router;
}
//# sourceMappingURL=express.js.map